import itertools
import json

from django.http import HttpResponse
from django.views.generic.base import TemplateView
from django.views.generic.detail import DetailView

from .models import Project, InstalledService, Module, InstalledVersion, Version
from .utils import CommitAndIssues, decorate_commit_line

class ProjectDetailView(DetailView):
    model = Project

    def get_context_data(self, **kwargs):
        context = super(ProjectDetailView, self).get_context_data(**kwargs)

        platforms = self.object.platform_set.all()
        services = self.object.service_set.all()

        context['platforms'] = platforms
        context['services'] = services

        return context


class ModuleDiffView(TemplateView):
    template_name = 'projects/diff.html'

    def get_context_data(self, name, commit1, commit2, **kwargs):
        context = super(ModuleDiffView, self).get_context_data(**kwargs)
        context['module'] = Module.objects.get(name=name)
        context['commit1'] = commit1
        context['commit2'] = commit2

        context['difflog'] = '\n'.join(
                [decorate_commit_line(line) for line in \
                        context['module'].get_diff_log(commit1, commit2).splitlines()])

        return context


class ModuleIssuesView(TemplateView):
    template_name = 'projects/issues.html'

    def get_context_data(self, name, commit1, commit2, **kwargs):
        context = super(ModuleIssuesView, self).get_context_data(**kwargs)
        context['module'] = Module.objects.get(name=name)
        context['commit1'] = commit1
        context['commit2'] = commit2
        commits = CommitAndIssues.get_for_commits(
                context['module'], commit1, commit2)
        issues = {}
        for commit in commits:
            for issue in commit.issues:
                if not int(issue.id) in issues:
                    issues[int(issue.id)] = issue
                else:
                    issue = issues[int(issue.id)]
                issue.add_commit(commit)

        issues_list = issues.items()
        issues_list.sort()
        context['issues'] = [x[1] for x in issues_list]

        return context


class ProjectHistoryView(DetailView):
    model = Project
    template_name_suffix = '_history'

    def get_context_data(self, **kwargs):
        context = super(ProjectHistoryView, self).get_context_data(**kwargs)

        installed_versions = InstalledVersion.objects.filter(
                service__platform__project=self.object).order_by('-timestamp')[:250]

        context['installed_versions'] = installed_versions

        return context



class ModulesView(TemplateView):
    template_name = 'projects/modules.html'

    def get_context_data(self, **kwargs):
        context = super(ModulesView, self).get_context_data(**kwargs)
        context['modules'] = Module.objects.all().order_by('name')
        return context

def try_int(x):
    try:
        return int(x)
    except ValueError:
        return 0

def cmp_version(v1, v2):
    return cmp([try_int(x) for x in v1.version.version.split('.')],
               [try_int(x) for x in v2.version.version.split('.')])


class ModuleDeploymentsView(TemplateView):
    template_name = 'projects/deployments.html'

    def get_context_data(self, name, **kwargs):
        context = super(ModuleDeploymentsView, self).get_context_data(**kwargs)
        context['versions'] = []
        context['module'] = Module.objects.get(name=name)
        module = context['module']
        for service in InstalledService.objects.all():
            version = module.get_installed_version(service.platform, service.service)
            if version and not version in context['versions']:
                if version.version.version:
                    context['versions'].append(version)

        context['versions'].sort(lambda x, y: cmp_version(x, y))
        return context


def module_deployments_json(request, name, **kwargs):
    response = HttpResponse(content_type='application/json')
    installed_versions = []
    module = Module.objects.get(name=name)
    for service in InstalledService.objects.all():
        version = module.get_installed_version(service.platform, service.service)
        if version and not version in installed_versions:
            installed_versions.append(version)

    json.dump([{'version': x.version.version,
                'project': x.service.platform.project.slug,
                'service': x.service.service.slug,
                'platform': x.service.platform.slug} for x in installed_versions],
                response)
    return response
